@@ -1,6 +1,6 @@
 /* sci.c -- System Configuration Interface
  *
- * Copyright (c) 1998  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
+ * Copyright (c) 1998/99  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
  *
@@ -11,6 +11,10 @@
  *   not to make any modifications and use it as it stands.
  *   
  * $Log: sci.c,v $
+ * Revision 1.12  1999/11/17 16:00:42  jab
+ * changed assembler to manually save registers, hopefully this should
+ * make the programs more stable
+ *
  * Revision 1.11  1999/07/25 14:39:49  jab
  * fixed bugs in SciOpenInterface and SciSetPassword
  * updated email address
@@ -47,23 +51,23 @@
  * Initial revision
  *
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 675 Mass Ave, Cambridge, MA 02139, USA.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software 
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  * 
  */
 
-static const char rcsid[]="$Id: sci.c,v 1.11 1999/07/25 14:39:49 jab Exp jab $";
+static const char rcsid[]="$Id: sci.c,v 1.12 1999/11/17 16:00:42 jab Exp jab $";
 
 #include<stdio.h>
 #include<stdlib.h>
@@ -84,10 +88,16 @@ int SciSupportCheck(int *version)
 	unsigned short dx;
 	unsigned char ah;
 
-	asm ("movw $0xf0f0,%%ax\n\t" \
+	asm ("pushl %%eax\n\t" \
+		"pushl %%ebx\n\t" \
+		"pushl %%ecx\n\t" \
+		"movw $0xf0f0,%%ax\n\t" \
 		"movw $0x0000,%%bx\n\t" \
 		"movw $0x0000,%%cx\n\t" \
 		"inb $0xb2,%%al\n\t" \
+		"popl %%ecx\n\t" \
+		"popl %%ebx\n\t" \
+		"popl %%eax\n" \
 		:"=dx" (dx), "=ah" (ah) : : "memory" );
 
 	*version = (int) dx;
@@ -105,11 +115,17 @@ int SciOpenInterface(void)
 {
 	unsigned char ah;
 
-	asm ("movw $0xf1f1,%%ax\n\t" \
+	asm ("pushl %%eax\n\t" \
+		"pushl %%ebx\n\t" \
+		"pushl %%ecx\n\t" \
+		"movw $0xf1f1,%%ax\n\t" \
 		"movw $0x0000,%%bx\n\t" \
 		"movw $0x0000,%%cx\n\t" \
 		"inb $0xb2,%%al\n\t" \
-		"movb %%ah,%0\n"
+		"movb %%ah,%0\n\t" \
+		"popl %%ecx\n\t" \
+		"popl %%ebx\n\t" \
+		"popl %%eax\n" \
 		:"=m" (ah) : : "memory" );
 
 	return (int) ah;
@@ -123,10 +139,16 @@ int SciCloseInterface(void)
 {
 	unsigned char ah;
 
-	asm ("movw $0xf2f2,%%ax\n\t" \
+	asm ("pushl %%eax\n\t" \
+		"pushl %%ebx\n\t" \
+		"pushl %%ecx\n\t" \
+		"movw $0xf2f2,%%ax\n\t" \
 		"movw $0x0000,%%bx\n\t" \
 		"movw $0x0000,%%cx\n\t" \
 		"inb $0xb2,%%al\n\t" \
+		"popl %%ecx\n\t" \
+		"popl %%ebx\n\t" \
+		"popl %%eax\n" \
 		:"=ah" (ah) : : "memory" );
 
 	return (int) (ah & 0xff);
@@ -145,8 +167,12 @@ int SciGet(SciRegisters *reg)
 	cx = reg->current;
 	dx = reg->possible;
 
-	asm ("movw $0xf3f3,%%ax\n\t" \
+	asm ("pushl %%esi\n\t" \
+		"pushl %%edi\n\t" \
+		"movw $0xf3f3,%%ax\n\t" \
 		"inb $0xb2,%%al\n\t" \
+		"popl %%edi\n\t" \
+		"popl %%esi\n" \
 		:"=ax" (ax), "=bx" (bx), "=cx" (cx), "=dx" (dx) \
 		:"bx" (bx), "cx" (cx) \
 		: "memory" );
@@ -183,8 +209,12 @@ int SciSet(SciRegisters *reg)
 	cx = reg->current;
 	dx = reg->possible;
 
-	asm ("movw $0xf4f4,%%ax\n\t" \
+	asm ("pushl %%esi\n\t" \
+		"pushl %%edi\n\t" \
+		"movw $0xf4f4,%%ax\n\t" \
 		"inb $0xb2,%%al\n\t" \
+		"popl %%edi\n\t" \
+		"popl %%esi\n" \
 		:"=ax" (ax) \
 		:"bx" (bx), "cx" (cx), "dx" (dx) \
 		: "memory" );
@@ -203,13 +233,21 @@ int SciACPower(void)
 	unsigned short cx;
 	unsigned char fl;
 
-	asm ("movw $0xfe00,%%ax\n\t" \
+	asm ("pushl %%eax\n\t" \
+		"pushl %%ebx\n\t" \
+		"pushl %%ecx\n\t" \
+		"movw $0xfe00,%%ax\n\t" \
 		"movw $0x0003,%%bx\n\t" \
 		"movw $0x0000,%%cx\n\t" \
 		"movw $0x0000,%%dx\n\t" \
 		"inb $0xb2,%%al\n\t" \
 		"lahf\n\t" \
-		:"=cx" (cx), "=ah" (fl) \
+		"movw %%cx,%0\n\t" \
+		"movb %%ah,%1\n\t" \
+		"popl %%ecx\n\t" \
+		"popl %%ebx\n\t" \
+		"popl %%eax\n" \
+		:"=m" (cx), "=m" (fl) \
 		: \
 		: "memory" );
 
