@@ -63,6 +63,8 @@
 extern "C" {
 #endif
 
+#ifndef SMMREGISTERS
+#define SMMREGISTERS
 typedef struct {
 	unsigned short ax;
 	unsigned short bx;
@@ -71,6 +73,7 @@ typedef struct {
 	unsigned int esi;
 	unsigned short di;
 } SMMRegisters;
+#endif
 
 typedef struct {
 	unsigned short setting;
@@ -105,6 +108,7 @@ enum {
 	SCI_BATTERY_TIME    = 0x0112,
 	SCI_LCD_BRIGHTNESS  = 0x0115,
 	SCI_BOOT_METHOD     = 0x011d,
+	SCI_2ND_BATTERY     = 0x011e,
 	SCI_CACHE_POLICY    = 0x011f,
 	SCI_COOLING_METHOD  = 0x0122,
 	SCI_STANDBY_TIME    = 0x0125,
